@@ -1,15 +1,19 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
+
 from builtins import input, range
 
-import os, sys, os.path, shlex
-from functools import wraps
+import os
+import os.path
+import shlex
+import sys
 from argparse import ArgumentTypeError
+from functools import wraps
 from typing import TYPE_CHECKING
 
+from .compat.os import makedirs
+from .compat.pathlib import Path
 from .iter import is_empty
 from .stdio import confirm
-from .compat.pathlib import Path
-from .compat.os import makedirs
 
 if TYPE_CHECKING:
 	from typing import Callable
@@ -62,7 +66,7 @@ def multiple_of(divisor):
 		error: argument --multiple: invalid int value: 'a'
 	"""
 	def int(s):
-		# type: (str, ) -> int
+		# type: (str, ) -> builtin_int
 
 		number = builtin_int(s)
 
@@ -80,7 +84,7 @@ def in_range(start, stop, step=1):
 	from builtins import int as builtin_int
 
 	def int(s):  # see: multiple_of()
-		# type: (str, ) -> int
+		# type: (str, ) -> builtin_int
 
 		number = builtin_int(s)
 
@@ -99,7 +103,7 @@ def between(start, stop):
 	from builtins import float as builtin_float
 
 	def float(s):
-		# type: (str, ) -> float
+		# type: (str, ) -> builtin_float
 
 		number = builtin_float(s)
 
@@ -223,9 +227,10 @@ def empty_dir(dirname):
 
 	""" tests if directory is empty """
 
-	if not is_empty(os.scandir(dirname)):
-		msg = "directory {0} is not empty".format(dirname)
-		raise ArgumentTypeError(msg)
+	with os.scandir(dirname) as it:
+		if not is_empty(it):
+			msg = "directory {0} is not empty".format(dirname)
+			raise ArgumentTypeError(msg)
 
 	return dirname
 
