@@ -1,18 +1,21 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
-from builtins import str, bytes
+
+from builtins import bytes, str
 from future.utils import PY2
 
 import os.path
-from tempfile import mkstemp
+from io import open
 from os import remove
 from sys import getfilesystemencoding
+from tempfile import mkstemp
 from typing import TYPE_CHECKING
 
+from .compat.os import fspath, replace
 from .file import copen
-from .compat.os import replace, fspath
 
 if TYPE_CHECKING:
-	from typing import Union, Optional, IO
+	from typing import IO, ContextManager, Optional, Union
+
 	from .compat.os import PathLike
 	PathType = Union[str, PathLike]
 
@@ -57,6 +60,14 @@ class TransactionalCreateFile(object):
 		else:
 			self.commit()
 
+def sopen(path, mode="rb", encoding=None, errors=None, newline=None, safe=False):
+	# type: (PathType, str, Optional[str], Optional[str], Optional[str], bool) -> ContextManager[IO]
+
+	if safe:
+		return TransactionalCreateFile(path, mode, encoding=encoding, errors=errors, newline=newline)
+	else:
+		return copen(path, mode, encoding=encoding, errors=errors, newline=newline)
+
 def write_file(data, path, mode="wb", encoding=None, errors=None, newline=None):
 	# type: (Union[str, bytes], PathType, str, Optional[str], Optional[str], Optional[str]) -> None
 
