@@ -1,13 +1,14 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 from future.utils import viewitems
+
 import logging
-from functools import wraps
 from collections import defaultdict
+from functools import wraps
 from typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
-	from typing import Callable, Collection, Optional, Any
+	from typing import Any, Callable, Collection, Optional
 
 def _to_union(types):
 	# type: (Collection[str], ) -> str
@@ -39,7 +40,7 @@ def rec_repr(obj):
 		return _type_str(obj)
 
 def _arg_str(arg, maxlen=None, app="...", repr_args=True):
-	# type: (Any, Optional[int], bool) -> str
+	# type: (Any, Optional[int], str, bool) -> str
 
 	if repr_args:
 		arg = repr(arg)
@@ -62,7 +63,12 @@ def _kwarg_str(key, value, maxlen=None, app="...", repr_args=True):
 def args_str(args, kwargs, maxlen=20, app="...", repr_args=True):
 	# type: (tuple, dict, Optional[int], str, bool) -> str
 
-	""" Creates printable string from function arguments. """
+	""" Creates printable string from function arguments.
+		If the string needs to be truncated to fit `maxlen`, `app` will be appended.
+		The length of `app` is not included in `maxlen`.
+		If the original string is shorter or equal `maxlen + len(app)`,
+		it will be returned unmodified.
+	"""
 
 	args = ", ".join(_arg_str(arg, maxlen, app, repr_args) for arg in args)
 	kwargs = ", ".join(_kwarg_str(k, v, maxlen, app, repr_args) for k, v in viewitems(kwargs))
