@@ -1,5 +1,12 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+	from typing import Any, Dict, Sequence, Set, Tuple, TypeVar, Union
+	T = TypeVar("T")
+	U = TypeVar("U")
+
 class ParseError(Exception):
 	""" Raised when the content to be parsed is malformed.
 		Not a value error because usually the error is in some external resource
@@ -64,6 +71,11 @@ class ExternalProcedureUnavailable(ExternalError):
 		correctly handled (or rather correctly not handled in thie case).
 	"""
 
+class DatabaseUnavailable(ExternalError):
+	""" Raised when the connection to a database fails because the database cannot be reached or
+		raises and error related to the database itself and not to the query.
+	"""
+
 # runtime, possible coding errors
 
 class ClosedObjectUsed(RuntimeError):
@@ -83,11 +95,14 @@ class MalformedFile(ValueError):
 		In contrast to `ParseError` this is usually a file we have control over.
 	"""
 
-def assert_choice(name, value, choices):
-	# type: (str, T, Set[T]) -> None
+def assert_choice(name, value, choices, optional=False):
+	# type: (str, T, Set[T], bool) -> None
+
+	if optional and value is None:
+		return
 
 	if value not in choices:
-		raise ValueError("{} must be one of {}".format(name, ", ".join(choices)))
+		raise ValueError("{} must be one of {}".format(name, ", ".join(map(str, choices))))
 
 def assert_choice_map(name, value, choices):
 	# type: (str, T, Dict[T, U]) -> U
@@ -95,13 +110,13 @@ def assert_choice_map(name, value, choices):
 	try:
 		return choices[value]
 	except KeyError:
-		raise ValueError("{} must be one of {}".format(name, ", ".join(choices.keys())))
+		raise ValueError("{} must be one of {}".format(name, ", ".join(map(str, choices.keys()))))
 
 def assert_type(name, value, types):
-	# type: (str, Any, Sequence[Any]) -> None
+	# type: (str, Any, Tuple[Union[type, Tuple[Any, ...]], ...]) -> None
 
 	if not isinstance(value, types):
-		raise TypeError("{} must be one of these types: {}".format(name, ", ".join(types)))
+		raise TypeError("{} must be one of these types: {}".format(name, ", ".join(map(str, types))))
 
 def assert_true(name, value):
 	if not value:
