@@ -1,23 +1,27 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 import os.path
-from sys import version_info, stdout
-from io import open, TextIOWrapper, TextIOBase, RawIOBase, BufferedIOBase, SEEK_SET, SEEK_END
-from typing import TYPE_CHECKING
+from io import SEEK_END, SEEK_SET, BufferedIOBase, RawIOBase, TextIOBase, TextIOWrapper, open
+from sys import stdout, version_info
+from typing import TYPE_CHECKING, overload
 
-from .iter import iter_equal, consume, resizer
+from .compat.os import PathLike, fdopen, fspath
+from .iter import consume, iter_equal, resizer
 from .math import PosInfInt
-from .ops import logical_xor, logical_implication
-from .compat.os import PathLike, fspath, fdopen
-
-if __debug__:
-	from .compat import gzip, bz2
-	import zipfile, tarfile
+from .ops import logical_implication, logical_xor
 
 if TYPE_CHECKING:
-	from typing import Callable, Optional, Union, IO, TextIO, BinaryIO, Iterator, Iterable
+	from mmap import mmap
+	from typing import IO, Any, BinaryIO, Callable, Iterable, Iterator, Optional, TextIO, TypeVar, Union
 
 	PathType = Union[str, PathLike]
+	Data = TypeVar("Data", str, bytes)
+
+if __debug__:
+	import tarfile
+	import zipfile
+
+	from .compat import bz2, gzip
 
 FILE_IO_BUFFER_SIZE = 8*1024*1024
 
@@ -42,7 +46,7 @@ def _stripmode(mode):
 	return "".join(set(mode) - {"t", "b"})
 
 def read_file(path, mode="b", encoding=None, errors=None):
-	# type: (PathType, str, Optional[str]) -> bytes
+	# type: (PathType, str, Optional[str], Optional[str]) -> bytes
 
 	""" Reads and returns whole file. If content is not needed use consume_file()"""
 
@@ -55,7 +59,7 @@ def read_file(path, mode="b", encoding=None, errors=None):
 		return fr.read()
 
 def write_file(data, path, mode="wb", encoding=None, errors=None):
-	# type: (Union[str, bytes], PathType, str, Optional[str]) -> None
+	# type: (Union[str, bytes], PathType, str, Optional[str], Optional[str]) -> None
 
 	""" Writes file. """
 
@@ -64,8 +68,19 @@ def write_file(data, path, mode="wb", encoding=None, errors=None):
 	with open(path, mode, encoding=encoding, errors=errors) as fw:
 		fw.write(data)
 
+@overload
+def read_or_raise(fin, size):
+	# type: (IO[Data], int) -> Data
+
+	pass
+
+@overload
+def read_or_raise(fin, size):
+	# type: (mmap, int) -> bytes
+
+	pass
+
 def read_or_raise(fin, size):
-	# type: (IO, int) -> Union[str, bytes]
 
 	""" Instead of returning a result smaller than `size` like `io.read()`,
 		it raises and EOFError.
@@ -98,7 +113,7 @@ def wrap_text(bf, mode, encoding, errors, newline):
 	return bf
 
 def copen(file, mode="rt", archive_file=None, encoding=None, errors=None, newline=None, compresslevel=9, ext=None):
-	# type: (Union[PathType, BinaryIO, int], str, Optional[str], Optional[str], Optional[str], Optional[str], int, Optional[str]) -> IO
+	# type: (Union[PathType, IO, int], str, Optional[str], Optional[str], Optional[str], Optional[str], int, Optional[str]) -> IO
 
 	""" Generic file open method. It supports transparent compression and improved text-mode handling.
 
@@ -198,7 +213,7 @@ class OptionalWriteOnlyFile(object):
 		# type: (Optional[PathType], str, Optional[str], Optional[str], Optional[str], int) -> None
 
 		if path:
-			self.fp = copen(path, mode, encoding=encoding, errors=errors, newline=newline, compresslevel=compresslevel)
+			self.fp = copen(path, mode, encoding=encoding, errors=errors, newline=newline, compresslevel=compresslevel)  # type: Optional[IO]
 		else:
 			self.fp = None
 
@@ -245,16 +260,18 @@ class StdoutFile(object):
 class PathOrBinaryIO(object):
 
 	def __init__(self, fname, mode="rb", close=False):
-		# type: (Union[PathType, BinaryIO],  str, str, str, Optional[str], bool) -> None
+		# type: (Union[PathType, BinaryIO], str, bool) -> None
 
 		if isinstance(fname, (RawIOBase, BufferedIOBase)):
-			self.doclose = close
-			self.fp = fname
+			self.doclose = close  # type: bool
+			self.fp = fname  # type: BinaryIO
 		else:
 			self.doclose = True
 			self.fp = copen(fname, mode)
 
 	def __enter__(self):
+		# type: () -> IO
+
 		return self.fp
 
 	def __exit__(self, exc_type, exc_value, traceback):
@@ -267,8 +284,8 @@ class PathOrTextIO(object):
 		# type: (Union[PathType, TextIO], str, str, str, Optional[str], bool) -> None
 
 		if isinstance(fname, TextIOBase):
-			self.doclose = close
-			self.fp = fname
+			self.doclose = close  # type: bool
+			self.fp = fname  # type: TextIO
 		else:
 			self.doclose = True
 			self.fp = copen(fname, mode, encoding=encoding, errors=errors, newline=newline)
@@ -432,8 +449,10 @@ class Tell(object):
 		return self._fp.writable()
 
 from future.moves.urllib import response
+
 from collections import deque
 
+
 class BufferedTell(response.addinfourl): # fixme: untested!!!
 
 	def __init__(self, filesize):
@@ -512,7 +531,7 @@ def copyfilelike(fin, fout, amount=None, buffer=FILE_IO_BUFFER_SIZE, report=None
 	return copied
 
 def simple_file_iter(fr, chunk_size=FILE_IO_BUFFER_SIZE):
-	# type: (IO, int) -> Iterator
+	# type: (IO[Data], int) -> Iterator[Data]
 
 	""" Iterate file-like object `fr` and yield chunks of size `chunk_size`. """
 
@@ -527,7 +546,7 @@ def simple_file_iter(fr, chunk_size=FILE_IO_BUFFER_SIZE):
 			break
 
 def reversed_file_iter(fp, chunk_size=FILE_IO_BUFFER_SIZE):
-	# type: (IO, int) -> Iterator
+	# type: (IO[Data], int) -> Iterator[Data]
 
 	""" Generate blocks of file's contents in reverse order. """
 
@@ -540,7 +559,7 @@ def reversed_file_iter(fp, chunk_size=FILE_IO_BUFFER_SIZE):
 		yield fp.read(delta)
 
 def limited_file_iter(fr, amount, chunk_size=FILE_IO_BUFFER_SIZE):
-	# type: (IO, int, int) -> Iterator
+	# type: (IO[Data], int, int) -> Iterator[Data]
 
 	""" Iterate file-like object `fr` and yield chunks of size `chunk_size`.
 		Limit output to `amount` bytes.
@@ -554,7 +573,7 @@ def limited_file_iter(fr, amount, chunk_size=FILE_IO_BUFFER_SIZE):
 		amount -= len(data)
 
 def iterfilelike(fr, start=0, amount=None, chunk_size=FILE_IO_BUFFER_SIZE):
-	# type: (IO, Optional[int], int) -> Iterator
+	# type: (IO[Data], int, Optional[int], int) -> Iterable[Data]
 
 	""" Iterate file-like object `fr` and yield chunks of size `chunk_size`.
 		Starts reading at `start` and optionally limit output to `amount` bytes.
@@ -569,7 +588,7 @@ def iterfilelike(fr, start=0, amount=None, chunk_size=FILE_IO_BUFFER_SIZE):
 		return limited_file_iter(fr, amount, chunk_size)
 
 def blockfileiter(path, mode="rb", encoding=None, errors=None, start=0, amount=None, chunk_size=FILE_IO_BUFFER_SIZE):
-	# type: (PathType, str, Optional[str], Optional[int], int) -> Iterator
+	# type: (PathType, str, Optional[str], Optional[str], int, Optional[int], int) -> Iterator[Data]
 
 	""" Iterate over file at `path` and yield chunks of size `chunk_size`.
 		Optionally limit output to `amount` bytes.
@@ -614,17 +633,17 @@ def blockfilesiter(paths, chunk_size=FILE_IO_BUFFER_SIZE):
 	if chunk:
 		yield chunk
 
-def bufferedfileiter(path, chunk_size, mode="rb", encoding=None, errors=None, amount=None, buffer_size=FILE_IO_BUFFER_SIZE):
-	# type: (PathType, int, str, Optional[str], Optional[int], int) -> Iterator
+def bufferedfileiter(path, chunk_size, mode="rb", encoding=None, errors=None, start=0, amount=None, buffer_size=FILE_IO_BUFFER_SIZE):
+	# type: (PathType, int, str, Optional[str], Optional[str], int, Optional[int], int) -> Iterable[Data]
 
 	""" Iterate over file at `path` reading chunks of size `buffer_size` at a time.
 		Yields data chunks of `chunk_size` and optionally limits output to `amount` bytes.
 	"""
 
-	return resizer(blockfileiter(path, mode, encoding, errors, amount, buffer_size), chunk_size)
+	return resizer(blockfileiter(path, mode, encoding, errors, start, amount, buffer_size), chunk_size)
 
 def byte_out(path, buffer_size=FILE_IO_BUFFER_SIZE):
-	# type: (PathType, int) -> Iterator[bytes]
+	# type: (PathType, int) -> Iterable[bytes]
 
 	""" Reads file at `path` byte by byte, using a read buffer of size `buffer_size`.
 	"""
@@ -639,18 +658,18 @@ def consume_file(filename, buffer_size=FILE_IO_BUFFER_SIZE):
 
 # was: same_files, textfile_equal: equal_files(*paths, mode="rt")
 def equal_files(*paths, **kwargs):
-	# type: (*PathType, str, int) -> bool
+	# type: (*PathType, **Any) -> bool
 
 	""" Check if files at `*paths` are equal. Chunks of size `chunk_size` are read at a time.
 		Data can be optionally limited to `amount`.
 	"""
 
 	# python2 fix for `equal_files(*paths, mode="rb", encoding=None, errors=None, amount=None, chunk_size=FILE_IO_BUFFER_SIZE)`
-	mode = kwargs.pop("mode", "rb")
-	encoding = kwargs.pop("encoding", None)
-	errors = kwargs.pop("errors", None)
-	amount = kwargs.pop("amount", None)
-	chunk_size = kwargs.pop("chunk_size", FILE_IO_BUFFER_SIZE)
+	mode = kwargs.pop("mode", "rb")  # type: str
+	encoding = kwargs.pop("encoding", None)  # type: Optional[str]
+	errors = kwargs.pop("errors", None)  # type: Optional[str]
+	amount = kwargs.pop("amount", None)  # type: Optional[int]
+	chunk_size = kwargs.pop("chunk_size", FILE_IO_BUFFER_SIZE)  # type: int
 	assert not kwargs, "Invalid keyword arguments"
 
 	its = tuple(blockfileiter(path, mode=mode, encoding=encoding, errors=errors, amount=amount, chunk_size=chunk_size) for path in paths)
@@ -670,7 +689,7 @@ def is_all_byte(fr, thebyte=b"\0", chunk_size=FILE_IO_BUFFER_SIZE):
 	return True
 
 def iter_zip(file, mode="rb", encoding=None, errors=None, newline=None, password=None):
-	# type: (Union[PathType, IO], str, Optional[str], Optional[str], Optional[str]) -> Iterator[IO]
+	# type: (Union[PathType, IO], str, Optional[str], Optional[str], Optional[str], Optional[bytes]) -> Iterator[IO]
 
 	"""
 		Iterate file-pointers to archived files. They are valid for one iteration step each.
@@ -690,7 +709,7 @@ def iter_zip(file, mode="rb", encoding=None, errors=None, newline=None, password
 					yield wrap_text(bf, mode, encoding, errors, newline)
 
 def iter_tar(file, mode="rb", encoding=None, errors=None, newline=None):
-	# type: (Union[PathType, IO], str, Optional[str], Optional[str], Optional[str]) -> Iterator[IO]
+	# type: (Union[PathType, BinaryIO], str, Optional[str], Optional[str], Optional[str]) -> Iterator[IO]
 
 	"""
 		Iterate file-pointers to archived files. They are valid for one iteration step each.
@@ -713,7 +732,9 @@ def iter_tar(file, mode="rb", encoding=None, errors=None, newline=None):
 	with cls as tf:
 		for ti in tf:
 			if ti.isfile(): # fixme: what about links?
-				with tf.extractfile(ti) as bf: # no `mode` for `extractfile`
+				bf = tf.extractfile(ti)
+				assert bf, "member `ti` is file, but still `extractfile` returned `None`"
+				with bf: # no `mode` for `extractfile`
 					yield wrap_text(bf, mode, encoding, errors, newline)
 
 def iter_lines(path, encoding="utf-8", errors="strict", newline=None, verbose=False):
