@@ -1,16 +1,21 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 
-import logging, re, mmap
-from struct import unpack
+import logging
+import mmap
+import re
 from io import open
+from struct import unpack
 from typing import TYPE_CHECKING
 
 from ..exceptions import ParseError
 from ..file import read_or_raise
+from ..string import backslash_escaped_ascii
 
 if TYPE_CHECKING:
-	from typing import BytesIO, Iterator, Tuple, Union
+	from typing import BinaryIO, Iterator, Set, Tuple, Union
+
 	from _hashlib import HASH as Hashobj
+	Segment = Union[Tuple[str, bytes], Tuple[str, bytes, bytes, bytes]]
 
 segments = {
 	# ISO/IEC 10918-1 : 1993(E), Table B.1
@@ -104,7 +109,7 @@ markerp = re.compile(b"\xFF[^\x00]")  # don't use raw literals
 metadata_segments = {"COM", "TRAILER", } | set("APP{}".format(i) for i in range(16))
 
 def iter_jpeg_fp(fr, translate=True):
-	# type: (BytesIO, bool) -> Iterator[Union[Tuple[str, bytes], Tuple[str, bytes, bytes, bytes]]]
+	# type: (BinaryIO, bool) -> Iterator[Segment]
 
 	""" Iterate over JPEG file given in binary stream `fr` and yield the segments.
 		`translate=False` can be used to reconstruct the file in a bit-identical way.
@@ -131,7 +136,7 @@ def iter_jpeg_fp(fr, translate=True):
 			try:
 				info = segments[marker]
 			except KeyError:
-				raise ParseError("Invalid segment marker: {} at {}".format(marker, mm.tell()))
+				raise ParseError("Invalid segment marker: {} at {}".format(backslash_escaped_ascii(marker), mm.tell()))
 
 			name = info[0]
 			hasdata = info[1]
@@ -147,7 +152,7 @@ def iter_jpeg_fp(fr, translate=True):
 					yield name, marker, size, data
 
 				prevenv = mm.tell()
-				for m in markerp.finditer(mm, prevenv, fs):
+				for m in markerp.finditer(mm, prevenv, fs):  # type: ignore
 					start, end = m.span(0)
 					if translate:
 						yield "ENTROPY", mm[prevenv:start]
@@ -160,7 +165,7 @@ def iter_jpeg_fp(fr, translate=True):
 					try:
 						info = segments[marker]
 					except KeyError:
-						raise ParseError("Invalid segment marker: {} at {}".format(marker, start))
+						raise ParseError("Invalid segment marker: {} at {}".format(backslash_escaped_ascii(marker), start))
 
 					name = info[0]
 
@@ -190,7 +195,7 @@ def iter_jpeg_fp(fr, translate=True):
 					yield name, marker, size, data
 
 def iter_jpeg(path, translate=True):
-	# type: (str, bool) -> Iterator[tuple]
+	# type: (str, bool) -> Iterator[Segment]
 
 	""" Same as `iter_jpeg_fp()` except that it accepts a path.
 	"""
@@ -200,7 +205,7 @@ def iter_jpeg(path, translate=True):
 
 
 def copy_jpeg_fp(fin, fout, ignore_segments=None):
-	# type: (BytesIO, BytesIO, Set[str]) -> None
+	# type: (BinaryIO, BinaryIO, Set[str]) -> None
 
 	""" Same as `copy_jpeg()` except that it accepts file-like objects.
 	"""
@@ -244,10 +249,10 @@ if __name__ == "__main__":
 	from argparse import ArgumentParser
 	from hashlib import sha1
 
-	from genutility.file import equal_files
 	from genutility.args import is_dir
-	from genutility.iter import consume
+	from genutility.file import equal_files
 	from genutility.hash import format_file_hash
+	from genutility.iter import consume
 
 	parser = ArgumentParser()
 	parser.add_argument("path", type=is_dir)
