@@ -2,15 +2,17 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 
 from builtins import filter, range
 
-import sys, os, logging
+import logging
+import os
 import subprocess  # nosec
+import sys
 from typing import TYPE_CHECKING
 
+from ..compat.os import scandir
+from ..compat.pathlib import Path
 from ..os import CurrentWorkingDirectory
 from ..string import surrounding_join
-from ..compat.pathlib import Path
-from ..compat.os import scandir
-from ..twothree.filesystem import tofs, fromfs
+from ..twothree.filesystem import fromfs, tofs
 
 if TYPE_CHECKING:
 	from typing import Callable
@@ -176,8 +178,9 @@ def create_rar_from_folder(path, dest_path=".", profile_setter_func=None, filter
 			r = Rar(dest_path / "{}.rar".format(name_transform(path.name)))
 			if profile_setter_func:
 				profile_setter_func(r)
-			for entry in filter(filter_func, scandir(".")): #was: scandir_rec
-				r.add_file(entry.path)
+			with scandir(".") as it:
+				for entry in filter(filter_func, it): #was: scandir_rec
+					r.add_file(entry.path)
 			r.create()
 		except RarError as e:
 			logger.error("%s\n%s" % (str(e), e.output))
