@@ -1,18 +1,26 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 
-import logging, os.path, sqlite3
+import logging
+import os.path
+import sqlite3
 from itertools import chain
+from typing import TYPE_CHECKING
 
 from tls_property import tls_property
 
 from .compat.os import fspath
 from .exceptions import NoResult
 from .filesystem import normalize_seps
-from .sql import iterfetch, fetchone
+from .sql import fetchone, iterfetch
 from .sqlite import quote_identifier
 
 logger = logging.getLogger(__name__)
 
+if TYPE_CHECKING:
+	from typing import Any, Dict, Iterable, Iterator, Optional, Set
+
+	from .filesystem import EntryType
+
 class GenericFileDb(object):
 
 	def __init__(self, dbpath, table, debug=True):
@@ -42,6 +50,10 @@ class GenericFileDb(object):
 	def trace(self, query):
 		logger.debug("SQL trace: %s", query)
 
+	@classmethod
+	def latest_order_by(cls):
+		return "entry_date DESC"
+
 	@classmethod
 	def primary(cls):
 		""" Never explicitly specified.
@@ -90,6 +102,8 @@ class GenericFileDb(object):
 		self.connection.close()
 
 	def setup(self):
+		# type: () -> None
+
 		fields = ", ".join("{} {}".format(n, t) for n, t, v in chain(self._primary, self._auto, self._mandatory, self._derived))
 
 		# unique = "UNIQUE({})".format(n for n, t, v in chain(self._mandatory, self._derived))
@@ -165,7 +179,7 @@ class GenericFileDb(object):
 			raise ValueError("No output fields selected")
 
 		conditions = " AND ".join("{} IS ?".format(n) for n, t, v in chain(self._mandatory, _derived))
-		order_by = "entry_date DESC"
+		order_by = self.latest_order_by()
 		sql = "SELECT {} FROM {} WHERE {} ORDER BY {} LIMIT 1".format(fields, self.table, conditions, order_by)  # nosec
 
 		self.cursor.execute(sql, args)
@@ -173,7 +187,7 @@ class GenericFileDb(object):
 		return fetchone(self.cursor)
 
 	def get(self, path, only=frozenset(), no=frozenset()):
-		# type: (Path, Set[str], Set[str]) -> tuple
+		# type: (EntryType, Set[str], Set[str]) -> tuple
 
 		""" Retrieves latest row based on mandatory information
 			which is solely based on the `path`.
@@ -214,7 +228,7 @@ class GenericFileDb(object):
 			return True
 
 	def add(self, path, derived=None, commit=True):
-		# type: (Path, Optional[Dict[str, Any]]) -> None
+		# type: (EntryType, Optional[Dict[str, Any]], bool) -> None
 
 		stats = path.stat()
 		self._add_file(fspath(path), stats.st_size, stats.st_mtime_ns, derived)
