@@ -3,26 +3,32 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 from builtins import chr, zip
 from future.moves.itertools import zip_longest
 
-import os, platform, stat, os.path, re, logging, shutil, errno
-from operator import attrgetter
+import errno
+import logging
+import os
+import os.path
+import platform
+import re
+import shutil
+import stat
 from fnmatch import fnmatch
 from functools import partial
+from operator import attrgetter
 from typing import TYPE_CHECKING
 
-from .file import equal_files, iterfilelike, FILE_IO_BUFFER_SIZE
-from .iter import is_empty
-from .os import islink, uncabspath, _not_available
-from .ops import logical_implication
-from .datetime import datetime_from_utc_timestamp
 from .compat import FileExistsError
-from .compat.os import PathLike, fspath, scandir
+from .compat.os import DirEntry, PathLike, fspath
 from .compat.pathlib import Path
+from .datetime import datetime_from_utc_timestamp
+from .file import FILE_IO_BUFFER_SIZE, equal_files, iterfilelike
+from .iter import is_empty
+from .ops import logical_implication
+from .os import _not_available, islink, uncabspath
 
 if TYPE_CHECKING:
-	from typing import Any, Callable, Optional, Iterator, Iterable, List, Set, Tuple, Union
 	from datetime import datetime
+	from typing import Any, Callable, Iterable, Iterator, List, Optional, Set, Tuple, Union
 	PathType = Union[str, PathLike]
-	from .compat.os import DirEntry
 	EntryType = Union[Path, DirEntry]
 
 logger = logging.getLogger(__name__)
@@ -50,10 +56,13 @@ class fileextensions(object):
 	images = ("bmp", "jpg", "jpeg", "png", "gif", "tif", "tiff", "tga")
 
 	archives = ("zip", "rar", "ace", "tar", "tgz", "tbz", "7z", "cab", "dmg", "wim")
-	executables = ("exe", "dll", "so", "scr", "msi", "pyd", "sys")
-	scripts = ("bat", "ps1", "sh", "py", "php")
-	documents = ("txt", "pdf", "doc", "docx", "ps")
+	bytecode = ("pyc", "class")
+	configuration = ("ini", "toml", "conf", "reg")
+	database = ("cat", "sqlite", "db")
+	documents = ("txt", "pdf", "doc", "docx", "ps", "rtf", "pgs")
+	executables = ("exe", "dll", "so", "scr", "msi", "pyd", "sys", "cpl")  # native code
 	image_archives = ("cbz", "cbr", "cb7", "cbt", "cba")
+	scripts = ("bat", "ps1", "sh", "c", "cpp", "cs", "py", "php", "js", "lua", "pl")  # and source code files
 
 	disc_images_binary = ("iso", "bin", "mdf", "img")
 	disc_images_sidecar = ("cue", "mds", "ccd")
@@ -62,9 +71,12 @@ class fileextensions(object):
 	game_images = ("gcn", "nds", "3ds", "wad", "xbx", "gba", "gb", "nes", "sfc", "n64", "z64")
 	compressed = ("gz", "bz2", "lzma", "xz", "z")
 	subtitles = ("srt", "sub", "idx", "sup", "ssa", "ass")
-
 	scene = ("nfo", "sfv")
 
+	windows = ("library-ms", "desklink", "lnk", "dmp")
+	macos = ("crash", )
+	os = windows + macos
+
 class IllegalFilename(ValueError):
 	pass
 
@@ -101,7 +113,7 @@ class FileProperties(object):
 	def __repr__(self):
 		# type: () -> str
 
-		args = ("{}={!r}".format(k, v) for k, v in zip(self.keys(), self.values.values()) if v is not None)
+		args = ("{}={!r}".format(k, v) for k, v in zip(self.keys(), self.values()) if v is not None)
 		return "FileProperties({})".format(", ".join(args))
 
 class DirEntryStub(object):
@@ -181,17 +193,22 @@ class MyDirEntry(BaseDirEntry):
 if TYPE_CHECKING:
 	MyDirEntryT = Union[DirEntry, MyDirEntry]
 
-def mdatetime(path):
-	# type: (PathType, ) -> datetime
+def mdatetime(path, aslocal=False):
+	# type: (PathType, bool) -> datetime
 
-	""" Returns the last modified date of `path`. """
+	""" Returns the last modified date of `path`
+		as a timezone aware datetime object.
 
-	if isinstance(path, Path):
+		If `aslocal=True` it will be formatted as local time,
+		and UTC otherwise (the default).
+	"""
+
+	if isinstance(path, (Path, DirEntry)):
 		mtime = path.stat().st_mtime
 	else:
 		mtime = os.stat(path).st_mtime
 
-	return datetime_from_utc_timestamp(mtime)
+	return datetime_from_utc_timestamp(mtime, aslocal)
 
 def rename(old, new):
 	# type: (PathType, PathType) -> None
@@ -267,7 +284,7 @@ def _scandir_rec_skippable(rootentry, files=True, others=False, follow_symlinks=
 	# type: (DirEntry, bool, bool, bool, Callable[[DirEntry, Exception], None]) -> Iterator[MyDirEntry]
 
 	try:
-		with scandir(rootentry.path) as it: # python 3.6
+		with os.scandir(rootentry.path) as it:
 			for entry in it:
 				if files and entry.is_file(follow_symlinks=follow_symlinks):
 					yield MyDirEntry(entry)
@@ -289,7 +306,7 @@ def _scandir_rec(rootentry, files=True, dirs=False, others=False, rec=True, foll
 	# type: (DirEntry, bool, bool, bool, bool, bool, Callable[[DirEntry, Exception], None]) -> Iterator[DirEntry]
 
 	try:
-		with scandir(rootentry.path) as it: # python 3.6
+		with os.scandir(rootentry.path) as it:
 			for entry in it:
 				if files and entry.is_file(follow_symlinks=follow_symlinks):
 					yield entry
@@ -344,15 +361,17 @@ def scandir_rec(path, files=True, dirs=False, others=False, rec=True, follow_sym
 def _scandir_depth(rootentry, depth, errorfunc):
 
 	try:
-		for entry in scandir(rootentry):
-			if entry.is_dir():
-				yield depth, entry
-				for depth_entry in _scandir_depth(entry, depth + 1, errorfunc):
-					yield depth_entry
+		with os.scandir(rootentry.path) as it:
+			for entry in it:
+				if entry.is_dir():
+					yield depth, entry
+					for depth_entry in _scandir_depth(entry, depth + 1, errorfunc):
+						yield depth_entry
 
-		for entry in scandir(rootentry):
-			if entry.is_file():
-				yield depth, entry
+		with os.scandir(rootentry.path) as it:
+			for entry in it:
+				if entry.is_file():
+					yield depth, entry
 
 	except OSError as e:
 		errorfunc(rootentry, e)
@@ -388,12 +407,12 @@ def make_writeable(path, stats=None):
 def is_writeable(stats):
 	# type: (os.stat_result, ) -> bool
 
-	return stats.st_mode & stat.S_IWRITE
+	return stats.st_mode & stat.S_IWRITE != 0
 
 def is_readable(stats):
 	# type: (os.stat_result, ) -> bool
 
-	return stats.st_mode & stat.S_IREAD
+	return stats.st_mode & stat.S_IREAD != 0
 
 def isfile(stats):
 	# type: (os.stat_result, ) -> bool
@@ -441,7 +460,7 @@ def _char_subber(s, illegal_chars, replacement):
 	return re.sub(regex, replacement, s)
 
 def _char_splitter(s, chars):
-	# type: (str, Set[str]) -> str
+	# type: (str, Iterable[str]) -> List[str]
 
 	regex = "[" + re.escape("".join(chars)) + "]"
 	return re.split(regex, s)
@@ -685,43 +704,47 @@ class Counts(object):
 		self.others = 0
 
 	def __iadd__(self, other):
-		# type: (Counts, ) -> None
+		# type: (Counts, ) -> Counts
 
 		self.dirs += other.dirs
 		self.files += other.files
 		self.others += other.others
 
+		return self
+
 	def null(self):
 		# type: () -> bool
 
 		return self.dirs == 0 and self.files == 0 and self.others == 0
 
 def _scandir_counts(rootentry, files=True, others=True, rec=True, total=False, errorfunc=scandir_error_raise):
-	# type: (MyDirEntryT, bool, bool, bool, bool, Callable) -> Iterator[DirEntry, Optional[Counts]]
+	# type: (MyDirEntryT, bool, bool, bool, bool, Callable) -> Iterator[Tuple[DirEntry, Optional[Counts]]]
 
 	counts = Counts()
 
 	try:
-		for entry in scandir(rootentry.path):
+		with os.scandir(rootentry.path) as it:
+			for entry in it:
 
-			if entry.is_dir():
-				counts.dirs += 1
-				if rec:
-					for subentry, subcounts in _scandir_counts(entry, files, others, rec, total, errorfunc):
-						yield subentry, subcounts
+				if entry.is_dir():
+					counts.dirs += 1
+					if rec:
+						for subentry, subcounts in _scandir_counts(entry, files, others, rec, total, errorfunc):
+							yield subentry, subcounts
 
-						if total:
-							counts += subcounts
+							if total:
+								assert subcounts
+								counts += subcounts
 
-			elif entry.is_file():
-				counts.files += 1
-				if files:
-					yield entry, None
+				elif entry.is_file():
+					counts.files += 1
+					if files:
+						yield entry, None
 
-			else:
-				counts.others += 1
-				if others:
-					yield entry, None
+				else:
+					counts.others += 1
+					if others:
+						yield entry, None
 
 		yield rootentry, counts  # yield after the loop
 
@@ -729,7 +752,7 @@ def _scandir_counts(rootentry, files=True, others=True, rec=True, total=False, e
 		errorfunc(rootentry, e)
 
 def scandir_counts(path, files=True, others=True, rec=True, total=False, onerror=scandir_error_log):
-	# type: (PathType, bool, bool, bool, bool, Callable) -> Iterator[DirEntry, Optional[Counts]]
+	# type: (PathType, bool, bool, bool, bool, Callable) -> Iterator[Tuple[DirEntry, Optional[Counts]]]
 
 	""" A recursive variant of scandir() which also returns the number of files/directories
 		within directories.
