@@ -1,30 +1,51 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 
+from builtins import map, range, reversed, zip
 from future.moves.itertools import zip_longest
 from future.utils import raise_from
-from builtins import zip, map, range, reversed
 
-import sys, logging, random
+import logging
+import random
+import sys
 from collections import deque
-from itertools import groupby, chain, tee, islice, count, combinations, product, starmap, repeat
-from operator import add
-from time import time, sleep
-from random import randrange
-from types import GeneratorType
 from functools import partial
+from itertools import chain, combinations, count, groupby, islice, product, repeat, starmap, tee
+from operator import add, itemgetter
+from random import randrange
+from time import sleep, time
+from types import GeneratorType
 from typing import TYPE_CHECKING
 
-from .exceptions import IteratorExhausted, EmptyIterable
+from .exceptions import EmptyIterable, IteratorExhausted
 from .ops import operator_in
 
 if TYPE_CHECKING:
-	from typing import (Any, Callable, Iterable, Iterator, TypeVar, Tuple, Sequence,
-		Optional, Collection, List, Union, Type, Dict, TextIO)
 	from logging import Logger
-	from .typing import SizedIterable, Number
+	from typing import (
+	    Any,
+	    Callable,
+	    Collection,
+	    Container,
+	    Dict,
+	    Iterable,
+	    Iterator,
+	    List,
+	    Optional,
+	    Sequence,
+	    Set,
+	    TextIO,
+	    Tuple,
+	    Type,
+	    TypeVar,
+	    Union,
+	)
+
+	from .typing import Number, Orderable, SizedIterable
 	T = TypeVar("T")
 	U = TypeVar("U")
 	V = TypeVar("V")
+	OrderableT = TypeVar("OrderableT", bound=Orderable)
+	ExceptionsType = Union[Type[Exception], Tuple[Type[Exception], ...]]
 
 logger = logging.getLogger(__name__)
 logger.addHandler(logging.NullHandler())
@@ -39,7 +60,7 @@ def iterrandrange(a, b):
 		yield randrange(a, b)  # nosec
 
 def repeatfunc(func, times=None, *args):
-	# type: (Callable[[*Any], T], Optional[int], *any) -> Iterator[T]
+	# type: (Callable[..., T], Optional[int], *Any) -> Iterator[T]
 
 	""" Repeat calls to func with specified arguments.
 		Example:  repeatfunc(random.random)
@@ -160,7 +181,7 @@ def flatten(it):
 			yield elm
 
 def diffsgn(a, b):
-	# type: (Number, Number) -> int
+	# type: (OrderableT, OrderableT) -> int
 
 	""" Sign of the finite derivative.
 		Equal to `-cmp(a, b)` or `cmp(b, a).
@@ -169,7 +190,7 @@ def diffsgn(a, b):
 	return (a < b) - (a > b)
 
 def asc_peaks(iterable):
-	# type: (Iterable[T], ) -> Iterator[T]
+	# type: (Iterable[OrderableT], ) -> Iterator[OrderableT]
 
 	""" Yields ascending peaks from `iterable`. """
 
@@ -186,7 +207,7 @@ def asc_peaks(iterable):
 	yield cur
 
 def extrema(iterable, first={1, -1}, last={1, -1}, derivatives={1, -1}):
-	# type: (Iterable[T], Set[int], Set[int], Set[int]) -> Iterator[T]
+	# type: (Iterable[OrderableT], Set[int], Set[int], Set[int]) -> Iterator[OrderableT]
 
 	it = iter(iterable)
 	try:
@@ -216,14 +237,14 @@ def extrema(iterable, first={1, -1}, last={1, -1}, derivatives={1, -1}):
 		yield old
 
 def peaks(it):
-	# type: (Iterable[T], ) -> Iterator[T]
+	# type: (Iterable[OrderableT], ) -> Iterator[OrderableT]
 
 	""" Yields peaks of `it`. """
 
 	return extrema(it, {-1}, {1}, {1})
 
 def valleys(it):
-	# type: (Iterable[T], ) -> Iterator[T]
+	# type: (Iterable[OrderableT], ) -> Iterator[OrderableT]
 
 	""" Yields valleys of `it`. """
 
@@ -263,6 +284,7 @@ def last(it):
 		return deque(it, 1).pop()
 	except IndexError:
 		raise_from(EmptyIterable("Empty iterable"), None)
+		raise  # just to appease mypy
 
 def batch(it, n, func=None):
 	# type: (Iterable[Any], int, Optional[Callable]) -> Iterator[Any] # return type cannot be more specific because of filter()
@@ -386,7 +408,7 @@ def consume(it):
 	deque(it, maxlen=0)
 
 def resizer(it, size, pad=False, pad_elm=None):
-	# type: (Iterable[Sequence[T]], int, bool, Optional[T]) -> Iterable
+	# type: (Iterable[Sequence[T]], int, bool, Optional[T]) -> Iterable[List[Optional[T]]]
 
 	""" Cuts the input iterable `it` consisting of variable length slices
 		into slices of length `size`. If pad is True, the last slice
@@ -400,12 +422,12 @@ def resizer(it, size, pad=False, pad_elm=None):
 		raise ValueError("size must be larger than 0")
 
 	it = iter(it)
-	buf = "" # type: Sequence[T]
+	buf = [] # type: Sequence[T]
 	pos = 0
 	try:
 		while True:
 			to_read = size
-			out = []
+			out = []  # type: List[Optional[T]]
 			while to_read > 0:
 				if len(buf) == pos:
 					buf = next(it)
@@ -452,7 +474,7 @@ def multi_join(it_a, it_b, join_func=add):
 		multi_join((1, 2), (3, 4), lambda x, y: str(x)+str(y)) -> ('13', '23', '14', '24')
 	"""
 
-	return [join_func(a, b) for b in it_b for a in it_a]
+	return (join_func(a, b) for b in it_b for a in it_a)
 
 def iter_except(iterator, exception_callbacks, return_on_exception=False):
 	# type: (Iterator[T], Dict[Type, Callable[[Iterator[T], Exception], bool]], bool) -> Iterator[T]
@@ -480,7 +502,7 @@ def iter_except(iterator, exception_callbacks, return_on_exception=False):
 				return
 
 def list_except(it, catch=Exception):
-	# type: (Iterable[T], Union[Exception, Sequence[Exception]]) -> Tuple[Exception, List[T]]
+	# type: (Iterable[T], ExceptionsType) -> Tuple[Optional[Exception], List[T]]
 
 	""" Same as `list()` except in the case of an exception, the partial list which was collected
 		so far is returned. `catch` specifies which exceptions are caught. It can be an exception
@@ -499,7 +521,7 @@ def list_except(it, catch=Exception):
 	return exc, ret
 
 def iter_except_ignore(iterator, exceptions, logger=logger):
-	# type: (Iterator[T], Collection[Exception], Logger) -> Iterator[T]
+	# type: (Iterator[T], ExceptionsType, Logger) -> Iterator[T]
 
 	""" Ignores `exceptions` raised in `iterator`. Does not work for Generators.
 	"""
@@ -516,7 +538,7 @@ def iter_except_ignore(iterator, exceptions, logger=logger):
 			logger.warning("Error in iterable: %s", str(e))
 
 def decompress(selectors, data, default=None):
-	# type: (Iterable[bool], Iterator[T], Optional[T]) -> Iterator[T]
+	# type: (Iterable[bool], Iterator[T], Optional[T]) -> Iterator[Optional[T]]
 
 	""" Basically the opposite of `itertools.compress`.
 		decompress([True, False, True], iter(["A", "B"])) -> ("A", None, "B")
@@ -539,7 +561,7 @@ def first_not_none(it, default=None):
 	return next(filternone(it), default)
 
 def range_count(start=0, stop=None, step=1):
-	# type: (int, Optional[int], int) -> Iterator[int]
+	# type: (int, Optional[int], int) -> Iterable[int]
 
 	""" Similar to `range`, except it optionally allows for an infinite range. """
 
@@ -578,7 +600,7 @@ def all_equal_to(it, cmp):
 	return all(elm == cmp for elm in it)
 
 def iter_equal(*its):
-	# type: (*Iterable, ) -> bool
+	# type: (*Iterable) -> bool
 
 	""" Test if the contents of all iterables `its` is the same.
 	"""
@@ -593,7 +615,7 @@ def iter_different(it_a, it_b):
 	return any(a != b for a, b in zip_longest(it_a, it_b))
 
 def every_n(it, n, pos=0):
-	# type: (Iterable[T], int, int) -> T
+	# type: (Iterable[T], int, int) -> Iterator[T]
 
 	""" Yields every `n`-th element from iterable `it` and starts at `pos`. """
 
@@ -615,7 +637,7 @@ def split(it, size):
 	return tuple(every_n(it, size, pos) for it, pos in zip(copies, range(size)))
 
 def no_dupes(*its):
-	# type: (*Iterable[T], ) -> Iterator[T]
+	# type: (*Iterable[T]) -> Iterator[T]
 
 	""" Merge `its` and removes all duplicates while preserving order. """
 
@@ -628,7 +650,7 @@ def no_dupes(*its):
 				seen.add(x)
 
 def retrier(waittime, attempts=-1, multiplier=1, jitter=0, max_wait=None, jitter_dist="uniform", waitfunc=sleep):
-	# type: (float, int, float, float, Optional[float], str, Callable[[float], Any]) -> Iterator[None]
+	# type: (float, int, float, float, Optional[float], str, Callable[[float], Any]) -> Iterator[int]
 
 	""" Iterator which yields after predefined amounts of time.
 		Supports `multiplier` to implements linear or exponential backoff.
@@ -637,9 +659,13 @@ def retrier(waittime, attempts=-1, multiplier=1, jitter=0, max_wait=None, jitter
 	"""
 
 	if jitter_dist == "uniform":
-		rand = random.uniform
+		def rand(waittime, jitter):
+			return waittime + random.uniform(-jitter, jitter)  # nosec
+
 	elif jitter_dist == "normal":
-		rand = random.normalvariate
+		def rand(waittime, jitter):
+			return random.normalvariate(waittime, jitter)
+
 	else:
 		raise ValueError("Unsupported jitter_dist")
 
@@ -657,7 +683,7 @@ def retrier(waittime, attempts=-1, multiplier=1, jitter=0, max_wait=None, jitter
 				waittime = max_wait
 				multiplier = 1
 
-			do_wait = max(0, waittime + rand(-jitter, jitter))
+			do_wait = max(0, rand(waittime, jitter))
 			waitfunc(do_wait)
 		else:
 			break
@@ -716,7 +742,7 @@ def any_in(it, container):
 	"""
 
 	if isinstance(it, set) and isinstance(container, set):
-		return not it.isdisjoint(set)
+		return not it.isdisjoint(container)
 
 	return any(elm in container for elm in it)
 
@@ -754,7 +780,7 @@ def select_by_indices(it, indices):
 		pass
 
 def skip_by_indices(it, indices):
-	# type: (Iterator[T], Iterator[int]) -> Iterator[int, T]
+	# type: (Iterator[T], Iterator[int]) -> Iterator[Tuple[int, T]]
 
 	i = 0
 	try:
@@ -771,3 +797,12 @@ def skip_by_indices(it, indices):
 
 	for ii, elem in enumerate(it, i):
 		yield ii, elem
+
+def remove_consecutive_dupes(it):
+	# type: (Iterable, ) -> Iterator
+
+	""" Removes consecutive duplicates from iterable `it`.
+		(a, a, b, b, a, b) -> a, b, a, b
+	"""
+
+	return map(itemgetter(0), groupby(it))
