@@ -3,16 +3,14 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 from builtins import range
 from future.utils import viewitems
 
-from math import sqrt, exp
-from typing import TYPE_CHECKING
+from math import exp, sqrt
+from typing import TYPE_CHECKING, Callable, Dict, Generic, Iterator, Optional, Tuple, TypeVar
 
 import numpy as np
 
 from .numba import opjit
 
-if TYPE_CHECKING:
-	from typing import Callable, Iterator, Tuple, TypeVar
-	T = TypeVar("T")
+T = TypeVar("T")
 
 RGB_YELLOW = (255, 255, 0)
 RGB_WHITE = (255, 255, 255)
@@ -21,6 +19,7 @@ def assert_square(name, value):
 	# type: (str, np.ndarray) -> None
 
 	""" Raises a ValueError if matrix `value` is not square.
+		value: [A, A]
 	"""
 
 	if not len(value.shape) == 2 or value.shape[0] != value.shape[1]:
@@ -125,10 +124,15 @@ def is_square(A):
 	return True
 
 def batch_vTAv(A, v):
-	# type: (float[B, X, X], float[B, X]) -> float[B]
+	# type: (np.ndarray, np.ndarray) -> np.ndarray
 
 	""" Performs batched calculation of `v^T A v` transform.
 		Special case of bilinear form `x^T A y`
+
+		A: float[B, X, X]
+		v: float[B, X]
+
+		returns: float[B]
 	"""
 
 	""" Faster than
@@ -228,7 +232,7 @@ class Sampler(object):
 		self.psum = cdf[-1]
 
 	def __call__(self):
-		# type: (int, ) -> int
+		# type: () -> int
 
 		""" Sample one. """
 
@@ -244,6 +248,8 @@ class Sampler(object):
 		return np.searchsorted(self.cdf, rands, side="right")
 
 	def pdf(self, n, minlength=None):
+		# type: (int, Optional[int]) -> np.ndarray
+
 		out = np.bincount(self.sample(n), minlength=minlength)
 		return out / n
 
@@ -256,24 +262,24 @@ def sample_probabilities(pvals):
 
 	return Sampler(np.cumsum(pvals))
 
-class UnboundedSparseMatrix(object):
+class UnboundedSparseMatrix(Generic[T]):
 
 	def __init__(self, dtype=float):
 		# type: (type, ) -> None
 
 		self.dtype = dtype
 		self.zero = self.dtype(0)
-		self.m = dict()
+		self.m = dict()  # type: Dict[Tuple[int, int], T]
 		self.cols = 0
 		self.rows = 0
 
 	def __getitem__(self, slice):
-		# type: (tuple, ) -> T
+		# type: (Tuple[int, int], ) -> T
 
 		return self.m.get(slice, self.zero)
 
 	def __setitem__(self, slice, value):
-		# type: (tuple, T) -> None
+		# type: (Tuple[int, int], T) -> None
 
 		c, r = slice
 		self.cols = max(self.cols, c+1)
