@@ -1,31 +1,43 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 
-import os, signal, logging
-from ctypes import byref, sizeof, create_unicode_buffer, WinError, cast, c_wchar_p, memset
-from ctypes.wintypes import ULARGE_INTEGER, LPCWSTR, DWORD
+import logging
+import os
+import signal
+from ctypes import WinError, byref, c_wchar_p, cast, create_unicode_buffer, memset, sizeof
+from ctypes.wintypes import DWORD, LPCWSTR, ULARGE_INTEGER
 from msvcrt import get_osfhandle
 from typing import TYPE_CHECKING
 
 from cwinsdk.shared.ehstorioctl import MAX_PATH
 from cwinsdk.shared.ntdef import PWSTR
-from cwinsdk.um.consoleapi import SetConsoleMode, GetConsoleMode, ENABLE_VIRTUAL_TERMINAL_PROCESSING
-from cwinsdk.um.fileapi import INVALID_FILE_ATTRIBUTES, GetDiskFreeSpaceExW, GetVolumeInformationW, LockFileEx, UnlockFileEx, GetFileAttributesW
-from cwinsdk.um.processenv import GetStdHandle
 from cwinsdk.um.combaseapi import CoTaskMemFree
-from cwinsdk.um.ShlObj_core import SHGetKnownFolderPath
+from cwinsdk.um.consoleapi import ENABLE_VIRTUAL_TERMINAL_PROCESSING, GetConsoleMode, SetConsoleMode
+from cwinsdk.um.fileapi import (
+    INVALID_FILE_ATTRIBUTES,
+    GetDiskFreeSpaceExW,
+    GetFileAttributesW,
+    GetVolumeInformationW,
+    LockFileEx,
+    UnlockFileEx,
+)
 from cwinsdk.um.KnownFolders import FOLDERID_RoamingAppData
 from cwinsdk.um.minwinbase import LOCKFILE_EXCLUSIVE_LOCK, LOCKFILE_FAIL_IMMEDIATELY, OVERLAPPED
+from cwinsdk.um.processenv import GetStdHandle
+from cwinsdk.um.ShlObj_core import SHGetKnownFolderPath
 from cwinsdk.um.WinBase import STD_OUTPUT_HANDLE
 from cwinsdk.um.winnt import FILE_ATTRIBUTE_REPARSE_POINT
 
-from .os_shared import _usagetuple, _volumeinfotuple
 from .compat.os import fspath
+from .os_shared import _usagetuple, _volumeinfotuple
 
 if TYPE_CHECKING:
-	from typing import IO
 	from ctypes.wintypes import HANDLE
+	from typing import IO, Union
+
 	from .compat.os import PathLike
 
+	PathType = Union[str, PathLike]
+
 unc_prefix = "\\\\?\\"
 
 def get_stdout_handle():
@@ -59,7 +71,7 @@ class EnableAnsi(object): # doesn't work for some reason...
 		self.close()
 
 def _islink(path):
-	# type: (PathLike, ) -> bool
+	# type: (PathType, ) -> bool
 
 	""" Tests if `path` refers to a symlink or a junction.
 		- Python >= 3.2 `os.path.islink()` only supports symlinks, not junctions.
@@ -75,7 +87,7 @@ def _islink(path):
 	return FileAttributes & FILE_ATTRIBUTE_REPARSE_POINT == FILE_ATTRIBUTE_REPARSE_POINT
 
 def _uncabspath(path):
-	# type: (str, )-> str
+	# type: (PathType, )-> str
 
 	if path.startswith(unc_prefix):
 		return os.path.abspath(path)
@@ -100,7 +112,7 @@ def _lock(fp, exclusive=True, block=False):
 	LockFileEx(handle, flags, 0, 0xffffffff, 0xffffffff, overlapped)
 
 def _unlock(fp):
-	# type: (IO, bool, bool) -> None
+	# type: (IO) -> None
 
 	fd = fp.fileno()
 	handle = get_osfhandle(fd)
@@ -128,6 +140,7 @@ def _get_appdata_dir():
 
 	ret = cast(Path, c_wchar_p).value
 	CoTaskMemFree(Path)
+	assert ret
 	return ret
 
 def _disk_usage_windows(path):
