@@ -1,12 +1,13 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 
+import logging
 import pickle  # nosec
-from pickle import HIGHEST_PROTOCOL  # nosec
-from functools import wraps
 from datetime import timedelta
+from functools import wraps
+from pickle import HIGHEST_PROTOCOL  # nosec
 from typing import TYPE_CHECKING
 
-from .atomic import TransactionalCreateFile
+from .atomic import sopen
 from .compat import FileNotFoundError
 from .compat.contextlib import nullcontext
 from .compat.os import fspath
@@ -17,9 +18,12 @@ from .object import args_to_key
 from .time import PrintStatementTime
 
 if TYPE_CHECKING:
-	from typing import Any, Callable, Optional, Iterator, Iterable
+	from typing import Any, Callable, Iterable, Iterator, Optional
+
 	from .compat.pathlib import Path
 
+logger = logging.getLogger(__name__)
+
 def read_pickle(path):
 	# type: (str, ) -> Any
 
@@ -39,12 +43,7 @@ def write_pickle(result, path, protocol=None, safe=False):
 		`safe`: if True, don't overwrite original file in case any error occurs
 	"""
 
-	if safe:
-		context = TransactionalCreateFile
-	else:
-		context = copen
-
-	with context(path, "wb") as fw:
+	with sopen(path, "wb", safe=safe) as fw:
 		pickle.dump(result, fw, protocol=protocol)
 
 def read_iter(path):
@@ -70,12 +69,7 @@ def write_iter(it, path, protocol=None, safe=False):
 			to actually write anything to disk.
 	"""
 
-	if safe:
-		context = TransactionalCreateFile
-	else:
-		context = copen
-
-	with context(path, "wb") as fw:
+	with sopen(path, "wb", safe=safe) as fw:
 		pickler = pickle.Pickler(fw, protocol=protocol)
 		for result in it:
 			pickler.dump(result)
