@@ -1,21 +1,22 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 from builtins import map, zip
-from future.utils import PY2, viewkeys, viewitems
+from future.utils import PY2, viewitems, viewkeys
 
 import re
+from collections import OrderedDict
+from functools import partial
 from itertools import chain
 from locale import strxfrm
-from functools import partial
-from collections import OrderedDict
 from typing import TYPE_CHECKING
 
+from .binary import decode_binary, encode_binary
 from .iter import switched_enumerate
-from .binary import encode_binary, decode_binary
 
 if TYPE_CHECKING:
-	from typing import Any, Dict, Iterable, Tuple, Callable, Optional, Sequence, TypeVar, Union, Mapping
+	from typing import Any, Callable, Dict, Iterable, Mapping, Optional, Sequence, Tuple, TypeVar, Union
 	T = TypeVar("T")
+	UnicodeOrdinalT = Union[int, str]
 
 english_consonants = "bcdfghjklmnpqrstvwxz" # y?
 english_vowels = "aeiouy" # y?
@@ -29,6 +30,29 @@ german_vowels = "aeiou"
 #def cycleright(s, start=0, end=-1): # bad
 #	return s[:start] + s[end] + s[start:end] + s[end+1:]
 
+def backslash_escaped_ascii(text):
+	# type: (bytes, ) -> str
+
+	return text.decode("ascii", "backslashreplace")
+
+def truncate(text, width, placeholder="..."):
+	# type: (str, int, str) -> str
+
+	""" Simple character based truncate method.
+		To truncate at word boundaries use `textwrap.shorten`.
+	"""
+
+	if width < 0:
+		raise ValueError("width cannot be negative")
+
+	if len(placeholder) > width:
+		raise ValueError("placeholder cannot be longer then width")
+
+	if len(text) <= width:
+		return text
+	else:
+		return text[0:width - len(placeholder)] + placeholder
+
 def toint(obj, default=None):
 	# type: (Any, Optional[T]) -> Union[int, Optional[T]]
 
@@ -47,7 +71,7 @@ def tryint(obj):
 	"""
 
 	try:
-		return int(obj)
+		return int(obj)  # type: ignore
 	except (ValueError, TypeError):
 		return obj
 
@@ -258,13 +282,13 @@ def replace_pairs_bytes(s, items):
 	return s.translate(table, delete)
 
 def replace_pairs_chars(s, items):
-	# type: (str, Dict[str, Optional[str]]) -> str
+	# type: (str, Dict[UnicodeOrdinalT, Optional[UnicodeOrdinalT]]) -> str
 
 	if PY2:
 		# table = s.maketrans(items) # 'unicode' object has no attribute 'maketrans'
 		table = {ord(k):v for k, v in viewitems(items)}
 	else:
-		table = s.maketrans(items)
+		table = s.maketrans(items)  # type: ignore # mypy issue #4374
 
 	return s.translate(table)
 
