@@ -2,25 +2,26 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 
 from builtins import range
 from future.moves.itertools import zip_longest
-from future.utils import PY2
+from future.utils import PY2, viewitems
 
-from unittest import TestCase
-from unittest.runner import TextTestRunner
-from unittest.result import TestResult
-from os import remove
 from collections import defaultdict
 from functools import wraps
+from io import TextIOWrapper
 from itertools import product
+from os import remove
 from tempfile import NamedTemporaryFile
 from time import sleep
-from io import TextIOWrapper
 from typing import TYPE_CHECKING
+from unittest import TestCase
+from unittest.result import TestResult
+from unittest.runner import TextTestRunner
 
-from .file import equal_files, _check_arguments
-from .signal import HandleKeyboardInterrupt # problem here
+from .file import _check_arguments, equal_files
+from .signal import HandleKeyboardInterrupt  # problem here
 
 if TYPE_CHECKING:
-	from typing import Any, Iterable, Optional, Hashable, Tuple, Callable, DefaultDict
+	from typing import Any, Callable, Collection, DefaultDict, Hashable, Iterable, Mapping, Optional, Tuple, TypeVar
+	T = TypeVar("T")
 
 class nullcontext(object): # see: .compat.contextlib
 
@@ -110,7 +111,7 @@ def make_comparable(d):
 	elif isinstance(d, tuple):
 		return tuple(make_comparable(i) for i in d)
 	elif isinstance(d, dict):
-		return make_comparable(list(d.items()))
+		return make_comparable(list(viewitems(d)))
 	else:
 		raise ValueError("must be list, tuple or dict, not {}".format(type(d)))
 
@@ -233,6 +234,15 @@ class MyTestCase(TestCase):
 
 		raise AssertionError("Result does not match one of the provided truths") # from None
 
+	def assertUnorderedMappingEqual(self, first, second, msg=None):
+		# type: (Mapping[T], Mapping[T], Optional[str]) -> None
+
+		self.assertEqual(len(first), len(second), msg)
+
+		for a, b in zip(sorted(first.keys()), sorted(second.keys())):
+			self.assertEqual(a, b)
+			self.assertEqual(first[a], second[b])
+
 def random_arguments(n, *funcs):
 	# type: (int, *Callable[[], Any]) -> Callable[[Callable], Callable]
 
